package nu.dll.nularspy;

import java.io.*;
import java.util.*;
import nu.dll.nular.Nular;

import org.jibble.socnet.*;

// Referenced classes of package nu.dll.nularspy:
//            NSConfiguration

public class NularSpy {

    File graphDataFile = new File("nularspy.dat");
    Nular nular;
    Graph graph;
    Set nicks;
    int entryCount;
    int pagesFetched = 0;
    int pageCount;
    Set beenThere;
    int depth;
    Configuration conf;


    public NularSpy(boolean initNular) throws IOException {
        nicks = new HashSet();
        entryCount = 0;
        beenThere = new HashSet();
        Properties properties = new Properties();
        properties.load(new FileInputStream("nularspy.ini"));
        conf = new NSConfiguration(properties);
	if (initNular) {
	    nular = new Nular(conf.getString("Username"), conf.getString("Password"));
	}
    }

    private void dprintln(String s) {
        System.out.println(new Date() + ": " + s);
    }

    private void build(String s, int i, int j) throws IOException {
        Node node = new Node(s);
        nicks.add(s);
        beenThere.add(s);
        graph.addNode(node);
        for(int k = 1; k <= pageCount; k++) {
	    StringBuffer buf = new StringBuffer();
	    for (int q=0; q < j; q++) {
		buf.append("\t");
	    }
            dprintln(buf.append(s + ", sida " + k + ", djup " + j).toString());
            List list = nular.getGuestbook(nular.getUserId(s), k);
	    pagesFetched++;
            Iterator iterator = list.iterator();
	    while (iterator.hasNext()) {

                Nular.GuestbookEntry guestbookentry = (Nular.GuestbookEntry) iterator.next();
                Node node1 = new Node(guestbookentry.author);
                nicks.add(guestbookentry.author);
                graph.addNode(node1);
                graph.addEdge(node1, node, 1.0D);
                entryCount++;

                if(!beenThere.contains(guestbookentry.author) && j < i)
                    build(guestbookentry.author, i, j + 1);

	    }
        }

    }

    public void buildGraph() throws IOException {
        buildGraph(nular.getMyUsername());
    }

    public void buildGraph(String s) throws IOException {
        buildGraph(s, conf.getInt("Depth"));
    }

    public void buildGraph(String s, int i) throws IOException {
        try {
            pageCount = conf.getInt("Pages");
	    depth = i;
            graph = new Graph("Lunarstorm", conf);
            graph.setTitle("Gstboksntverk med start hos \"" + nular.getMyUsername() + "\"");
            Node node = new Node(nular.getMyUsername());
            graph.addNode(node);
            build(s, i, 0);
	} catch (IOException ex1) {
	    dprintln("I/O-fel: " + ex1.toString());
	    ex1.printStackTrace();
        } finally {
            dprintln(nicks.size() + " anvndare, " + entryCount + " inlgg.");
	    dprintln(pagesFetched + " sidor hmtade.");
	    boolean purgeLeafs = true;
	    if (purgeLeafs) {
		dprintln("Rensar...");
		graph.purgeLeafs();
	    }
	    saveGraph();
        }
    }

    public void saveGraph() throws IOException {
	ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(graphDataFile));
	oos.writeInt(depth);
	oos.writeInt(pageCount);
	oos.writeObject(graph);
	oos.close();
    }

    public void readGraph() throws IOException {
	try {
	    ObjectInputStream ois = new ObjectInputStream(new FileInputStream(graphDataFile));
	    depth = ois.readInt();
	    pageCount = ois.readInt();
	    graph = (Graph) ois.readObject();
	    ois.close();
	    graph.setConfiguration(conf);
	} catch (ClassNotFoundException ex1) {
	    throw new IOException("Unexpected object class: " + ex1.toString());
	}
    }

    public void generateImage() throws IOException {
	dprintln("Genererar bild...");
	graph.setDescription1(graph.getNodeCount() + " anvndare, djup: " + depth + ", historik: " + pageCount);
	graph.setDescription2("Fler gstboksinlgg ger tjockare linje.");
	graph.setDescription3("Baserat p http://www.jibble.org/piespy/ - skapad " + new Date());
	graph.makeNextImage();
	dprintln("klart.");
    }

    public static void main(String args[]) throws Exception {
	NularSpy ns;
	if (args.length > 0 && args[0].equals("-fetch")) {
	    ns = new NularSpy(true);
	    ns.buildGraph();
	} else {
	    ns = new NularSpy(false);
	    ns.readGraph();
	}
	ns.generateImage();
    }

}
